"""
Helper functions to parse FlockLab link quality output files for use with TriScale.
"""

import csv
import os
import numpy as np
import pandas as pd

def parse_data_file(    data_file,
                        active_link_threshold=None,
                        verbose=False):

    """
    Parses the publically available FlockLab link quality data [1].

    Adapted from the `parse_flocklab_results.ipynb` notebook, available with
    the data [1].

    Parameters
    ----------
    data : string
        Path to and name of a FlockLab link quality preprocessed csv,
        available in [1].
    active_link_threshold : integer or None, optional
        Minimal number of received packets between two nodes to consider
        the link as "active".
        Default to 50 and print a notification to the user.
    verbose : True/False, optional
        When true, print non-functional and intermediary outputs.
        Default : False

    Returns
    -------
    df : DataFrame
        Pandas DataFrame with DatetimeIndex and a `link_quality` column
        containing the mean link quality across all active links (links with
        at least `active_link_threshold` packets received).

    References
    ----------
    .. [1] R. Jacob et al., "Wireless Link Quality Estimation on FlockLab - and
        Beyond ", (Version v1.1.4) [Data set]. Zenodo.
        http://doi.org/10.5281/zenodo.3408382
    """

    todo = ''
    todo += '# ---------------------------------------------------------------- \n'
    todo += '# TODO parse_data_file \n'
    todo += '# ---------------------------------------------------------------- \n'
    todo += '# ---------------------------------------------------------------- \n'

    if verbose:
        print('%s' % todo)

    ##
    # Checking the inputs
    ##

    if not isinstance(data_file, str):
        raise ValueError("Wrong input type. Expect a string, got "+repr(data_file)+".")

    if active_link_threshold is None:
        active_link_threshold = 50
        print('Active link threshold not specified; set to %g' % active_link_threshold)
    else:
        if (    not isinstance(active_link_threshold, int) or
                active_link_threshold >= 100 or
                active_link_threshold <= 0):
            raise ValueError("""Wrong input.
            'active_link_threshold' must be a integer between 0 and 100.""")

    ##
    # Load and parse input data
    ##

    # Open the data file and load it in a DataFrame
    df = pd.read_csv(data_file, delimiter=',')

    # Parse dates
    df['date_time'] = pd.to_datetime(df['date_time'], utc=True)
    date_time = df.date_time.unique()

    # Compute the average link quality
    df.rename(columns={'rcv_total':'link_quality'}, inplace=True)
    df.loc[df['link_quality'] < active_link_threshold] = np.nan
    link_quality = df.groupby('date_time')['link_quality'].mean()
    # Create a new DataFrame
    df = pd.DataFrame(data={'link_quality':link_quality})

    return df

def parse_serial_log_glossy_use_case(   data_file,
                                        node_list,
                                        verbose=True):

    """
    Parses FlockLab test results from collected for a wireless use case
    for TriScale [1]. It produces two csv files
    * glossy_reliability_N1.csv
    * glossy_reliability_N2.csv
    suitable for futher analysis using the TriScale API.

    All test results (raw data) are available on TriScale artefact
    repository [2].

    Adapted from the `parse_flocklab_results.ipynb` notebook [3].

    Parameters
    ----------
    data_file : string
        Path to and name of a FlockLab test serial log file ('serial.csv')
        available in [2].
    node_list : list
        Node ids used in the test
    verbose : True/False, optional
        When true, print non-functional and intermediary outputs.
        Default : False

    Returns
    -------
    None

    References
    ----------
    .. [1] Anonymous, "TriScale: A Framework Supporting Reproducible Networking
        Evaluations", Submitted to NSDI, 2020,
        https://doi.org/10.5281/zenodo.3464273
    .. [2] Anonymous, "TriScale - Source code and related artifacts", Zenodo,
        2020, http://doi.org/10.5281/zenodo.3451418
    .. [3] R. Jacob et al., "Wireless Link Quality Estimation on FlockLab - and
        Beyond ", (Version v1.1.4) [Data set]. Zenodo.
        http://doi.org/10.5281/zenodo.3408382
    """

    if not isinstance(data_file, str):
        raise ValueError("Wrong input type. First parameter must be a string, got "+repr(data_file)+".")

    # Set the result file name
    result_file_N1 = data_file[:-10]+'glossy_reliability_N1.csv'
    result_file_N2 = data_file[:-10]+'glossy_reliability_N2.csv'

    # Create the result files and write the header line
    with open(result_file_N1, 'w',newline='') as csvfile:
        result_writer = csv.writer(csvfile, delimiter=',', quotechar='|', quoting=csv.QUOTE_MINIMAL)
        result_writer.writerow([
            'node_id',
            'prr',
        ])

    with open(result_file_N2, 'w',newline='') as csvfile:
        result_writer = csv.writer(csvfile, delimiter=',', quotechar='|', quoting=csv.QUOTE_MINIMAL)
        result_writer.writerow([
            'node_id',
            'prr',
        ])

    # Open the test serial log
    f = open( data_file , "r")

    # Initialized list of failed nodes during for this test
    failed_node_list = []

    # Loop through the list of nodes
    for node_id in node_list:

        # Re-start reading the serial file from the top
        f.seek(0, 0)

        # Reset tracking counters
        data_to_read = 2

        snd_counter = len(node_list)
        # -> To stop parsing the file once all data from `node_id` has been read
        scheduled_to_send = 0
        # -> Mark that the `node_id` has been transmitting
        slot_failed_counter = 0
        # -> Track the number of failed slots

        # Read serial log first line
        line = f.readline()

        while line != '' and data_to_read != 0:

            # Discard the header line
            if line[0]=='#':
                line = f.readline()
                continue

            tmp = line[0:-1].split(',')

            if int(tmp[2]) == node_id:

                # Parse the data from the serial log
                if "Log:" in tmp[4]:

                    if "Tot" in tmp[4]:
                        (tag, tag2, rcv_total, tag3, channel) = tmp[4].split(':')
                        prr = int( int(rcv_total) / (len(node_list) - 1) * 100 )
                        # Save test data in tmp file
                        if data_to_read == 2:
                            # First read the N=1 data
                            out_file = result_file_N1
                        else:
                            # Then read the N=2 data
                            out_file = result_file_N2

                        with open(out_file, 'a', newline='') as csvfile:
                            result_writer = csv.writer(csvfile,
                                                       delimiter=',',
                                                       quotechar='|',
                                                       quoting=csv.QUOTE_MINIMAL)
                            result_writer.writerow([
                                node_id,
                                prr,
                            ])

                        data_to_read -= 1

            # Read the next lime
            line = f.readline()

    if verbose:
        print("Done: %s" % data_file)

    return
